/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Event;

import com.google.gwt.event.shared.GwtEvent;

import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;


/**
 * Event used to ask current view to refresh
 * 
 * @author Jean-Yves Goubet
 *
 */
public class ViewRefreshEvent extends GwtEvent<ViewRefreshHandler> {

  // the id of the view to refresh
  private String    elementId = null;
  
  // the type of the view to refresh
  private PTab.Type elementType = null;
  
  // this boolean is used to know if this event is fired because of switching tab or not (for focus panel in finder)
  private boolean   focus = false;

  
  /**
   * Constructor of the event
   * 
   * @author Jean-Yves Goubet
   * @param elementId : the id of the view to refresh
   * @param elementType : the type of the view to refresh
   * @param focus : the focus boolean
   * 
   */
  public ViewRefreshEvent(String elementId, PTab.Type elementType, boolean focus) {
    super();
    //
    this.elementId = elementId;
    this.elementType = elementType;
    this.focus = focus;
  }
  
  
  public static final Type<ViewRefreshHandler> TYPE = new Type<ViewRefreshHandler>();
  
  
  protected void dispatch(ViewRefreshHandler handler) {
    handler.onRefresh(this);    
  }

  
  public com.google.gwt.event.shared.GwtEvent.Type<ViewRefreshHandler> getAssociatedType() {
    return TYPE;
  }
  
  
  /**
   * Getter of the elementId parameter
   * 
   * @author Jean-Yves Goubet
   * @return String : the elementId value
   */
  public String getElementId() {
    return this.elementId;
  }
  
  
  /**
   * Getter of the elementType parameter
   * 
   * @author Jean-Yves Goubet
   * @return String : the elementType value
   */
  public PTab.Type getElementType() {
    return this.elementType;
  }
  
  
  /**
   * Getter of the focus parameter
   * 
   * @author Jean-Yves Goubet
   * @return String : the focus value
   */
  public boolean getFocus() {
    return this.focus;
  }

}
