/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Class that contains the presenter part of ListInstance.
 * 
 * @author Jean-Yves Goubet
 * 
 */
public class PListInstance extends PresenterIF {
 
  private VListInstanceIF view;
  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  private String processId = null;
  private PTabLayoutPanel pTabLayoutPanel = null;
  
  private HandlerManager eventBus = null;
  private ViewRefreshHandler refreshHandler = null;
  
  /**
   * The constructor of the class
   * 
   * @author Jean-Yves Goubet
   * @param li : interface representing the view of the class
   * @param pTabLayoutPanel 
   *
   */
  public PListInstance(VListInstanceIF vListInstance, PTabLayoutPanel pTabLayoutPanel, HandlerManager eventBus) {
    this.bindingView(vListInstance);
    //
    this.pTabLayoutPanel = pTabLayoutPanel;

    //
    this.eventBus = eventBus;
    this.addListeners();

    this.view.getListInstanceGrid().setHandlerManager(eventBus);
    this.view.getListInstanceGrid().setPresenterTabLayoutPanel(pTabLayoutPanel);
    this.view.getListInstanceGrid().setDebugId();
    //
    this.view.getListInstanceGrid().setOrchestraManagementServices(orchestramanagement);
    this.view.getListInstanceGrid().setStringLimitSize(10, 15, "...");
  }
  

  /**
   * This constructor is used when adding a tab for a specific process.
   * 
   * @author Jean-Yves Goubet
   * @param processId : the id of the process selected
   * @param pTab : the tab presenter (to set its child widget)
   * 
   */
  public PListInstance(PTabLayoutPanel pTabLayoutPanel, String processId, String elementName, HandlerManager eventBus) {
    VListInstance vListInstance = new VListInstance();
    this.bindingView(vListInstance);
    pTabLayoutPanel.addTab(processId, elementName, PTab.Type.LIST_INSTANCE, vListInstance, this);
    //
    this.processId = processId;
    this.pTabLayoutPanel = pTabLayoutPanel;
    
    //
    this.eventBus = eventBus;
    this.addListeners();

    this.view.getListInstanceGrid().setHandlerManager(eventBus);
    this.view.getListInstanceGrid().setPresenterTabLayoutPanel(pTabLayoutPanel);    
    this.view.getListInstanceGrid().setDebugId();
    //
    this.view.getListInstanceGrid().setOrchestraManagementServices(orchestramanagement);
    this.view.getListInstanceGrid().setStringLimitSize(10, 15, "...");
    this.view.getListInstanceGrid().setProcessId(processId);
    // first refresh
    this.refreshCurrentList();    
  }
  
  
  protected void addListeners() {
    addViewRefreshHandlerToBus();
  }
  
  /**
   * The handler to get events to launch refreshes
   * 
   * @author Jean-Yves Goubet
   * @param eventBus : the eventBus to listen
   */
  private void addViewRefreshHandlerToBus() {

    refreshHandler = new ViewRefreshHandler() {
      public void onRefresh(ViewRefreshEvent viewRefreshEvent) {
        if (viewRefreshEvent.getElementType() == PTab.Type.LIST_INSTANCE) {
          if (viewRefreshEvent.getElementId() == null) {
            if (PListInstance.this.processId == null) {
              PListInstance.this.refreshCurrentList();
            }
          } else if (viewRefreshEvent.getElementId().equals(PListInstance.this.processId)) {
            PListInstance.this.refreshCurrentList();
          }
        }
      }
    };
    
    eventBus.addHandler(ViewRefreshEvent.TYPE, refreshHandler); 
  }
  
  
  /**
   * 
   */
  protected void refreshCurrentList() {
    this.view.getListInstanceGrid().refreshCurrentList();  
  }


  /**
   * Attach our interface of the view with the real class.
   * 
   * @author Jean-Yves Goubet
   * @param lp : interface that represents the view of the class
   * 
   */
  protected void bindingView(final Object lp) {
    this.view = (VListInstanceIF) lp;
  }

  /**
   * Get method of the view.
   * 
   * @author Jean-Yves Goubet
   * 
   */    
  public VListInstanceIF getView() {
    return view;
  }
  
  
  protected void setDebugID() {
    
  }

  /**
   * Destroy the ListInstance object. This function remove the handler add on the eventBus in
   * the constructor.
   * 
   * @author Mathieu Crassin
   * 
   */
  public void destroy() {
    if (eventBus != null) {
      if (refreshHandler != null) {
        eventBus.removeHandler(ViewRefreshEvent.TYPE, refreshHandler);
      }
    }
  }
}

