/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ProcessDetails;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;

import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionClass;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.ProcessGrid;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Presenter of the Process Details class
 * 
 * @author Jean-Yves Goubet
 *
 */
public class ProcessDetailsGrid extends SpecialGrid {
  
  private OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);
  
  private String processId = "";
  
  private int idLimitSize = 10;
  private int nameLimitSize = 10;
  private int namespaceLimitSize = 30;

  private PTabLayoutPanel presenterTabLayoutPanel = null;

  private HandlerManager eventBus = null;
  
  private static String []columnTitles = {"UUID","Name","Namespace","Deployment Date","Undeployment Date","State","Actions"};
  
  /**
   * Constructor of the presenter
   * 
   * @author Jean-Yves Goubet
   * @param view : the view of the Process Details class
   */
  public ProcessDetailsGrid() {
    //
    super(columnTitles, 2, false);
  }
  
  public void setProcessId(String processId) {
    this.processId = processId;
  }

  private void setDebugId() {
    this.getWidget(0,0).ensureDebugId("processIdLabel-UUID-" + processId + "-PA");
    this.getWidget(0,1).ensureDebugId("processNameLabel-UUID-" + processId + "-PA");
    this.getWidget(0,2).ensureDebugId("processNamespaceLabel-UUID-" + processId + "-PA");
    this.getWidget(0,3).ensureDebugId("processDeploymentDateLabel-UUID-" + processId + "-PA");
    this.getWidget(0,4).ensureDebugId("processUndeploymentDateLabel-UUID-" + processId + "-PA");
    this.getWidget(0,5).ensureDebugId("processStateLabel-UUID-" + processId + "-PA");    
    this.getWidget(0,6).ensureDebugId("processActionsLabel-UUID-" + processId + "-PA");
    
    
    this.getWidget(1,0).ensureDebugId("processIdValue-UUID-" + processId + "-PA");
    this.getWidget(1,1).ensureDebugId("processNameValue-UUID-" + processId + "-PA");
    this.getWidget(1,2).ensureDebugId("processNamespaceValue-UUID-" + processId + "-PA");
    this.getWidget(1,3).ensureDebugId("processDeploymentDateValue-UUID-" + processId + "-PA");
    this.getWidget(1,4).ensureDebugId("processUndeploymentDateValue-UUID-" + processId + "-PA");
    this.getWidget(1,5).ensureDebugId("processStateValue-UUID-" + processId + "-PA");
    this.getWidget(1,6).ensureDebugId("processActionsValue-UUID-" + processId + "-PA");
    
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(0).getElement(), "instancesImageID-UUID-" + processId + "-PA"); 
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(1).getElement(), "activitiesImageID-UUID-" + processId + "-PA");
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(2).getElement(), "undeployImageID-UUID-" + processId + "-PA"); 
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(3).getElement(), "removeImageID-UUID-" + processId + "-PA");   
  }
  
  
  /**
   * This method is used to add a process in the processes list displayed.
   * 
   * @author Jean-Yves Goubet
   * @param process : it's the process to add to the grid
   * 
   */
  protected void addProcess(final MProcess process) {
    // add data to the end of the grid
    int rowIndex = 1;
    //instantiate labels
    RowLabel[] rowLabels = new RowLabel[columnTitles.length - 1];
    rowLabels[0] = formatLabel(process.getProcessId(), idLimitSize, rowIndex);
    rowLabels[1] = formatLabel(process.getProcessName(), nameLimitSize, rowIndex);
    rowLabels[2] = formatLabel(process.getProcessNamespace(), namespaceLimitSize, rowIndex);
    rowLabels[3] = new RowLabel(ProcessGrid.dateFormatter(process.getProcessDeploymentDate()),rowIndex,false);
    rowLabels[4] = (process.getProcessState().toString().equals("undeployed")) ? new RowLabel(ProcessGrid.
        dateFormatter(process.getProcessUndeploymentDate()),rowIndex,false) : new RowLabel("N/A",rowIndex,false);
    rowLabels[5] = new RowLabel(process.getProcessState(),rowIndex,false);
    //addHandler to labels and add labels to grid
    addData(rowLabels);
    addActionsToProcess(process.getProcessState());
    // define the Debug Id
    this.setDebugId();
  }  

  
  /**
   * This method is used to fill in the actions column depending on process' state.
   * 
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process concerned
   * @param state : the state of the process
   * 
   */
  private void addActionsToProcess(String state) {
    ActionImage [] tab = new ActionImage[4];
    // instances action
    tab[0] = new ActionImage(ImageResources.INSTANCE.instancesIcon(),
        ImageResources.INSTANCE.instancesIconDisabled(), 1);
    tab[0].setTitle("instances");
    
    // activities action
    tab[1] = new ActionImage(ImageResources.INSTANCE.activities(),
        ImageResources.INSTANCE.activitiesDisabled(), 1);
    tab[1].setTitle("activities");
    tab[1].setEnabled(false);
    
    // undeploy action
    tab[2] = new ActionImage(ImageResources.INSTANCE.undeployIcon(), 
        ImageResources.INSTANCE.undeployIconDisabled(), 1);
    tab[2].setTitle("undeploy");
    
    // delete action 
    tab[3] = new ActionImage(ImageResources.INSTANCE.deleteIcon(), 
        ImageResources.INSTANCE.deleteIconDisabled(), 1);
    tab[3].setTitle("remove");
    
    this.addActions(tab, state, 6);
  }  
  
  
  /**
   * 
   */
  protected void addClickHandlers(final ActionImage[] tabImages, String state) {
    
    // instances
    tabImages[0].setHandler(new ActionClass() {
      public void execute() {
        // create list instance presenter 
        new PListInstance(presenterTabLayoutPanel,      // also give the tabLayoutPanel (thus the presenter could manage tab)
            ProcessDetailsGrid.this.processId, // and to finish, give the process Id
            ProcessDetailsGrid.this.getText(tabImages[0].getRowIndex(), 1), 
            eventBus
        );
      }
    });
    
    // activities (not possible)
    
    // undeployed
    if (state.equals("undeployed")) {
      tabImages[2].setEnabled(false); 
      tabImages[3].setEnabled(true);
      
      tabImages[3].setHandler(new ActionClass() {
        public void execute() {
          removeProcess();
        }
      });       
    }
    // deployed
    else if (state.equals("deployed")) {
      tabImages[2].setEnabled(true);
      tabImages[3].setEnabled(false);
      
      tabImages[2].setHandler(new ActionClass() {
        public void execute() {
          undeployProcess();
        }
      });      
    }
  }
  
  
  /**
   * This method is used to call the undeployProcess method on orchestra management servlet.
   * 
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process
   * 
   */
  private void removeProcess() {
    this.orchestramanagement.removeProcess(processId, 
        new AsyncCallback<Void>() {
          public void onSuccess(Void arg0) {
            // delete list instance of a process tab if exists
            ProcessDetailsGrid.this.presenterTabLayoutPanel.removeTab(ProcessDetailsGrid.this.processId,
                PTab.Type.LIST_INSTANCE);
            // delete also activity list tab if exists
            ProcessDetailsGrid.this.presenterTabLayoutPanel.removeTab(ProcessDetailsGrid.this.processId,
                PTab.Type.LIST_PROCESS_ACTIVITY);
            ProcessDetailsGrid.this.eventBus.fireEvent(
                new ViewRefreshEvent(ProcessDetailsGrid.this.processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(Throwable caught) {
            GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while removing process !", caught.getMessage());
            }
          }
        });
  }   
  
  
  /**
   * This method is used to call the undeployProcess method on orchestra management servlet.
   * 
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process
   * 
   */
  private void undeployProcess() {
    this.orchestramanagement.undeployFromOrchestra(processId, 
        new AsyncCallback<Void>() {
          public void onSuccess(Void arg0) {
            ProcessDetailsGrid.this.eventBus.fireEvent(
                new ViewRefreshEvent(ProcessDetailsGrid.this.processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(Throwable caught) {
            GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while undeploy file !", caught.getMessage());
            }
          }
        });
  }
  
  /**
   * To get info on the process from Orchestra server.
   * 
   * @author Jean-Yves Goubet
   */
  public void refreshCurrentList() {
//System.out.println("ProcessDetailsGrid - processId = " + processId);

    orchestramanagement.getProcess(processId, new AsyncCallback<MProcess>() {    
      public void onSuccess(MProcess process) {
        ProcessDetailsGrid.this.manageGrid();
        addProcess(process);
        ProcessDetailsGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(Throwable caught) {
        GWT.log("ProcessDetailsGrid - refreshCurrentList: onFailure : " + caught.getMessage());
        ProcessDetailsGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
   
  }
  
  public void setPresenterTabLayoutPanel(PTabLayoutPanel pTabLayoutPanel) {
    this.presenterTabLayoutPanel  = pTabLayoutPanel;
  }

  public void setHandlerManager(HandlerManager eventBus) {
    this.eventBus  = eventBus;
  }

  /**
   * This function is empty because the sort functionality is inactive on this grid.
   * But we have to implement it because this function belong to the SpecialGrid class. 
   * 
   * @author Mathieu Crassin
   */  
  public void sortAndDisplayList() {
  }
}
