/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Widget;

import java.util.ArrayList;

import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.DeployProcess.PDeployProcess;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerHandler;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Finder.PFinder;
import org.ow2.orchestra.console.GWTConsole.client.InfoBar.PInfoBar;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.PListProcess;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.Tab.VTab;


public class PTabLayoutPanel {
  
  /**
   * @author Mathieu Crassin
   *
   * The interface that represent the view
   */
  private VTabLayoutPanelIF view = null;
  private HandlerManager eventBus = null;
  
  private final int refreshInterval = 5000;
  private Timer refreshTimer = null;
  protected boolean refreshTimerWorks;
  
  protected ArrayList<PresenterId> presenterList = null;
  
  private class PresenterId {
    private final String elementId;
    private final PTab.Type elementType;
    private final PresenterIF presenter;
    
    public PresenterId(final String elementId, final PTab.Type elementType, final PresenterIF presenter) {
      this.elementId = elementId;
      this.elementType = elementType;
      this.presenter = presenter;
    }
    
    public String getElementId() {
      return elementId;
    }
    public PTab.Type getElementType() {
      return elementType;
    }    
    public PresenterIF getPresenter() {
      return presenter;
    }
  }
  
  public PTabLayoutPanel(HandlerManager eventBus) {
    //
    this.eventBus = eventBus;
    //
    VTabLayoutPanel vTabLayoutPanel = new VTabLayoutPanel();
    //
    this.bindingView(vTabLayoutPanel);
    //
    PInfoBar pInfoBar = new PInfoBar(this.view.getLayoutPanel(), eventBus);
    //
    this.createPresenters();
    //
    this.addTimer();
    this.addCloseWindowListener();
    this.addListener(pInfoBar);
    this.addSuspendTimerHandlerToBus();
  }

  
  /**
   * 
   */
  private void addSuspendTimerHandlerToBus() {
    this.eventBus.addHandler(SuspendTimerEvent.TYPE, new SuspendTimerHandler() {
      public void onSuspend(SuspendTimerEvent suspendTimerEvent) {
        if (suspendTimerEvent.getSuspend()) {
          refreshTimer.cancel();
          refreshTimerWorks  = false;
        } else {
          String refresh = Window.Location.getParameter("refresh");
          if (refresh != null && refresh.equals("true")) {
            refreshTimer.scheduleRepeating(refreshInterval);
            refreshTimerWorks = true;
          }
        }
      }
    });
  }


  /**
   * 
   */
  private void addCloseWindowListener() {
    if (getUserAgent().contains("msie")) {
      Window.addCloseHandler(new CloseHandler<Window>() {
        public void onClose(CloseEvent arg0) {         
          refreshTimer.cancel();
        }
      });
    }
  }


  /**
   * Add the PTabLayoutPanel timer that refresh the active tab
   * 
   * @author Mathieu Crassin
   */
  private void addTimer() {
    
    refreshTimer = new Timer() {
      @Override
      public void run() {
        PTabLayoutPanel.this.eventBus.fireEvent(
            new ViewRefreshEvent(((VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(
                PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex())).getElementId(),
            ((VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(
                PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex())).getElementType(), true));
      }
    };
    
    String res = Window.Location.getParameter("refresh");

    if (res != null && res.equals("true")
        || res == null) {
    
      System.out.println("refresh active");
      
      refreshTimer.scheduleRepeating(refreshInterval);
      refreshTimerWorks = true;
    } else {
      System.out.println("refresh inactive");
    }
  }

  
  /** 
   * Add selection tab listener
   * 
   * @author Mathieu Crassin
   */
  private void addListener(PInfoBar pInfoBar) {
    this.view.getTabLayoutPanel().addSelectionHandler(new SelectionHandler<Integer>() {
      public void onSelection(SelectionEvent<Integer> arg0) {
        String refresh = Window.Location.getParameter("refresh");
        if (refreshTimerWorks || (refresh != null && refresh.equals("false"))) {
          PTabLayoutPanel.this.eventBus.fireEvent(
              new ViewRefreshEvent(((VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(
                  PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex())).getElementId(),
              ((VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(
                  PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex())).getElementType(), false));
        }
      }
    });
    //
    pInfoBar.getView().getRefreshLabel().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent arg0) {
        eventBus.fireEvent(new SuspendTimerEvent(false));
        VTab vTab = (VTab) PTabLayoutPanel.this.view.getTabLayoutPanel().getTabWidget(PTabLayoutPanel.this.view.getTabLayoutPanel().getSelectedIndex());
        eventBus.fireEvent(new ViewRefreshEvent(vTab.getElementId(), vTab.getElementType(), true));
      }     
    });
  }
  
  /**
   * @author Mathieu Crassin
   * @param dp interface that represent the view of our application
   *
   * Attach our interface of the view with the real class
   */
  private void bindingView(final VTabLayoutPanelIF dp) {
    this.view = dp;
  }  
  
  /**
   * Create the presenter of the view containing in this panel.
   * 
   * @author Mathieu Crassin
   */
  private void createPresenters() {
    PListProcess plp = new PListProcess(this.view.getVListProcess(), this, this.eventBus);
    new PListInstance(this.view.getVListInstance(), this, this.eventBus);
    new PDeployProcess(this.view.getVDeployProcess(),plp, this.eventBus);
    new PFinder(this, this.view.getVFinder(), this.eventBus);
    this.addTabID();
  }
  
  

  /**
   * Define Id for the 2 first tab 
   * 
   * @author Jean-Yves Goubet
   */
  private void addTabID() {
    this.view.getTabLayoutPanel().getTabWidget(0).ensureDebugId("tabLayoutPanelID-0");
    this.view.getTabLayoutPanel().getTabWidget(1).ensureDebugId("tabLayoutPanelID-1");
    this.view.getTabLayoutPanel().getTabWidget(2).ensureDebugId("tabLayoutPanelID-2");
  } 
  
  
  /**
   * Function that manage the add tab task. 
   * First the function check if the tab already exists and
   * then create the presenter & the view of the tab.
   * 
   * @author Mathieu Crassin
   * @param processID
   * @param processName
   */
  public void addTab(String elementId, String elementName, PTab.Type type, Widget widget, PresenterIF presenter) {
    // check tab already exist
    int res = tabAlreadyExists(elementId, type);
    
    // tab doesn't exists
    if (res == -1) {
      // add the new tab
      PTab pres = new PTab(elementName, elementId, type, widget, presenter, this.view.getTabLayoutPanel());
      
      // keep a pointer to the presenter
      addPresenter(elementId, type, pres);
      
      // select it
      this.view.getTabLayoutPanel().selectTab(this.view.getTabLayoutPanel().getWidgetCount() - 1);      
    }
    // else
    else {
      // move it to the first pos.
      if (res >= 2) {
        // select it
        this.view.getTabLayoutPanel().selectTab(res);        
      }
    }      
  }

  
  /**
   * Function that search in the existing process tab list a process.  
   * 
   * @param processID
   * @param processName
   * @return
   */
  public int tabAlreadyExists(String processID, PTab.Type type) {
    // this method is probably slow, and could be replace by a search in a list
    int i = 2, indexTab = -1;
    while (i < this.view.getTabLayoutPanel().getWidgetCount() && indexTab == -1) {
      // cast the result of the getTabWidget (the header)
      VTab tmp = (VTab)this.view.getTabLayoutPanel().getTabWidget(i);
      // compare with the research param.
      if (processID.equals(tmp.getElementId()) && type == tmp.getElementType()) {
        indexTab = i;
      }
      
      i++;
    }
    
    return indexTab;
  }

  
  /**
   * This method is called to close tab if its corresponding process is undeployed.
   * 
   * @param elementId : UUID of the element to remove.
   * @param type : type of the element to remove.
   * 
   * @author Jean-Yves Goubet
   */
  public void removeTab(String elementId, PTab.Type type) {
    int indexTab = tabAlreadyExists(elementId, type);
    if (indexTab != -1) {
      // get the presenter that match with the view 
      PresenterIF pres = findPresenter(elementId, type);
      // call the destroy method
      pres.destroy();
      // remove presenter from the list 
      removePresenter(elementId, type);
      
      this.view.getTabLayoutPanel().remove(indexTab);
    }
  }
  
  /**
   * Get a string that represente the browser name.
   * 
   * @author Mathieu Crassin
   * @return
   * 
   */
  private static native String getUserAgent() /*-{
    return navigator.userAgent.toLowerCase();
  }-*/;


  /**
   * 
   * @return
   */
  public VTabLayoutPanelIF getView() {
    return this.view;
  }
  

  /**
   * Method to get the height of content displayed
   * 
   * @return 
   */
  public int getHeightContent() {
    return this.view.getTabLayoutPanel().getWidget(this.view.getTabLayoutPanel().getSelectedIndex()).getOffsetHeight() + 174;
  }
  
  
  
  
  public void addPresenter(final String elementId, final PTab.Type elementType, PresenterIF presenter) {
 // System.out.println("ADD PRESENTER TO LIST");    

     PresenterId presId = new PresenterId(elementId, elementType, presenter);
     if (presenterList == null) {
       presenterList = new ArrayList<PresenterId>();      
     }
     
     if (recherchePresenter(elementId, elementType) == -1) {
       presenterList.add(presId);
     } 
     
     System.out.println("Nb = " + presenterList.size());    
   }
   
   public PresenterIF findPresenter(final String elementId, final PTab.Type elementType) {
 // System.out.println("FIND PRESENTER id = " + elementId + " & name = " + elementType);    

     for (int i = 0; i < presenterList.size(); i++) {
       PresenterId tmp = presenterList.get(i);
       
       if (tmp.getElementId().equals(elementId)
           && tmp.getElementType().equals(elementType)) {

         return tmp.getPresenter();
       }
     }

     return null;
   }
   public void removePresenter(final String elementId, final PTab.Type elementType) {
     int i = recherchePresenter(elementId, elementType);
     presenterList.remove(i);
   }
   private int recherchePresenter(final String elementId, final PTab.Type elementType) {
     int i = 0;
     boolean found = false;
     while (i < presenterList.size() && !found) {
       PresenterId tmp = presenterList.get(i);
       
       if (tmp.getElementId().equals(elementId)
           && tmp.getElementType().equals(elementType)) {
         
         found = true;
       } else {
         i++;
       }
     }   
     
     if (found) {
       return i;
     } else {
       return -1;
     }
   }  
}
